from sqlalchemy.ext.asyncio import AsyncSession, create_async_engine
from sqlalchemy.orm import sessionmaker
from src.db.config import settings

engine = create_async_engine(settings.connection_string, echo=True)


AsyncSessionLocal = sessionmaker(
    bind=engine, class_=AsyncSession, autoflush=False, expire_on_commit=False
)


async def get_db():
    session = AsyncSessionLocal()
    try:
        yield session
    finally:
        await session.close()



from src.db.base import Base
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy import ForeignKey, String, Date
from datetime import date as py_date

class PassportData(Base):
    __tablename__ = "passport_data"

    id: Mapped[int] = mapped_column(primary_key=True)
    user_id: Mapped[int] = mapped_column(ForeignKey("users.id"), nullable=False)
    passport_series_number: Mapped[str] = mapped_column(String(20))
    issue_date: Mapped[py_date] = mapped_column(Date)
    issuing_authority: Mapped[str] = mapped_column(String(100))
    authority_code: Mapped[str] = mapped_column(String(10))
    place_of_birth: Mapped[str] = mapped_column(String(150))
    date_of_birth: Mapped[py_date] = mapped_column(Date)
    gender: Mapped[str] = mapped_column(String(10))
    nationality: Mapped[str] = mapped_column(String(50))

    user: Mapped["User"] = relationship(back_populates="passport_data")


from src.db.base import Base
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy import ForeignKey, String

class AdditionalInfo(Base):
    __tablename__ = "additional_info"

    id: Mapped[int] = mapped_column(primary_key=True)
    user_id: Mapped[int] = mapped_column(ForeignKey("users.id"), nullable=False)
    registration_address: Mapped[str] = mapped_column(String(255), nullable=False)
    actual_address: Mapped[str] = mapped_column(String(255), nullable=False)
    extra_phones: Mapped[str] = mapped_column(String(255), nullable=False)
    email: Mapped[str] = mapped_column(String(255), nullable=False)
    privileges: Mapped[str] = mapped_column(nullable=False)
    military_service: Mapped[str] = mapped_column(nullable=True)

    user: Mapped["User"] = relationship(back_populates="additional_info")


__all__ = (
    "Faculty",
    "Direction",
    "Specialization",
    "StudyForm",
    "Application",
    "User",
    "AdditionalInfo",
    "EducationInfo",
    "PassportData"
)

from .application import Application
from .faculty import Faculty
from .direction import Direction
from .specialization import Specialization
from .study_form import StudyForm

from .user import User
from  .education_info import EducationInfo
from .additional_info import AdditionalInfo
from .passport_data import PassportData



from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    DB_USER: str
    DB_HOST: str
    DB_PASSWORD: str
    DB_PORT: str
    DB_NAME: str

    ACCESS_SECRET_KEY: str
    REFRESH_SECRET_KEY: str

    ACCESS_TOKEN_EXPIRE_MINUTES: int
    REFRESH_TOKEN_EXPIRE_DAYS: int

    ALGORITHM: str

    @property
    def connection_string(self) -> str:
        return (
            f"postgresql+asyncpg://"
            f"{self.DB_USER}:"
            f"{self.DB_PASSWORD}@"
            f"{self.DB_HOST}:{self.DB_PORT}/"
            f"{self.DB_NAME}"
        )

    model_config = SettingsConfigDict(env_file=".env")


settings = Settings()



__all__ = (
    "Faculty",
    "Direction",
    "Specialization",
    "StudyForm",
    "Application",
    "User",
    "AdditionalInfo",
    "EducationInfo",
    "PassportData"
)

from src.models import Application
from src.models import Faculty
from src.models import Direction
from src.models import Specialization
from src.models import StudyForm

from src.models import User
from  src.models import EducationInfo
from src.models import AdditionalInfo
from src.models import PassportData


from src.db.base import Base
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy import ForeignKey

class Application(Base):
    __tablename__ = "applications"

    id: Mapped[int] = mapped_column(primary_key=True)
    user_id: Mapped[int] = mapped_column(ForeignKey("users.id"), nullable=False)
    faculty_id: Mapped[int] = mapped_column(ForeignKey("faculties.id"), nullable=False)
    direction_id: Mapped[int] = mapped_column(ForeignKey("directions.id"), nullable=False)
    specialization_id: Mapped[int] = mapped_column(ForeignKey("specializations.id"), nullable=False)
    study_form_id: Mapped[int] = mapped_column(ForeignKey("study_forms.id"), nullable=False)
    funding_type: Mapped[str] = mapped_column(nullable=False)
    status: Mapped[str] = mapped_column(nullable=False, default='pending')

    user: Mapped["User"] = relationship(back_populates="applications")
    faculty: Mapped["Faculty"] = relationship(back_populates="applications")
    direction: Mapped["Direction"] = relationship(back_populates="applications")
    specialization: Mapped["Specialization"] = relationship(back_populates="applications")
    study_form: Mapped["StudyForm"] = relationship(back_populates="applications")


from src.db.base import Base
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy import ForeignKey, String, Date
from datetime import date

class EducationInfo(Base):
    __tablename__ = "education_info"

    id: Mapped[int] = mapped_column(primary_key=True)
    user_id: Mapped[int] = mapped_column(ForeignKey("users.id"), nullable=False)
    document_type: Mapped[str] = mapped_column(String(100), nullable=False)
    document_number: Mapped[str] = mapped_column(String(50), nullable=False)
    issue_date: Mapped[date] = mapped_column(Date)
    institution_name: Mapped[str] = mapped_column(String(255), nullable=False)
    average_score: Mapped[str] = mapped_column(String(20), nullable=False)

    user: Mapped["User"] = relationship(back_populates="education_info")


from setuptools import setup, find_packages

setup(
    name="sharq_models",
    version="0.1.0",
    packages=find_packages(),
    install_requires=[
        "alembic==1.16.2",
        "asyncpg==0.30.0",
        "SQLAlchemy==2.0.41",
        "pydantic==2.11.7",
        "pydantic-settings==2.9.1",
        "python-dotenv==1.1.0",
    ],  
    author="Bekzod",
    author_email="pterest160@gmail.com",
    description="A package for Sharq project models",  # Update this with your repo URL
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",  # or your license
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)



from src.db.base import Base
from sqlalchemy.orm import Mapped, mapped_column, relationship

class StudyForm(Base):
    __tablename__ = "study_forms"

    id: Mapped[int] = mapped_column(primary_key=True)
    name: Mapped[str] = mapped_column(nullable=False)

    applications: Mapped[list["Application"]] = relationship(back_populates="study_form")


from src.db.base import Base
from sqlalchemy.orm import Mapped, mapped_column, relationship

class User(Base):
    __tablename__ = "users"

    id: Mapped[int] = mapped_column(primary_key=True)
    first_name: Mapped[str] = mapped_column(nullable=False)
    last_name: Mapped[str] = mapped_column(nullable=False)
    username: Mapped[str] = mapped_column(nullable=True, unique=True)
    password: Mapped[str] = mapped_column(nullable=True)
    phone_number: Mapped[str] = mapped_column(nullable=False, unique=True)

    passport_data: Mapped["PassportData"] = relationship(back_populates="user", uselist=False)
    applications: Mapped[list["Application"]] = relationship(back_populates="user")
    education_info: Mapped[list["EducationInfo"]] = relationship(back_populates="user")
    additional_info: Mapped[list["AdditionalInfo"]] = relationship(back_populates="user")